<?php
/**
*
* @package JVA - phpBB Shoutbox
* @version $Id$
* @author 2011-2020 KillBill
* @copyright (c) 2014-2020 https://jv-arcade.com/ - https://jv-arcade.com/contact
* @license https://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace jv\shoutbox\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class listener implements EventSubscriberInterface
{
	protected $container, $request, $db, $auth, $user, $config, $shout, $php_ext, $shout_table, $posts_table;
	protected $arcade_enable, $arcade;

	public function __construct($container, $request, $db, $auth, $user, $config, $shout, $arcade = null, $php_ext, $shout_table, $posts_table)
	{
		$this->container = $container;
		$this->request = $request;
		$this->db = $db;
		$this->auth = $auth;
		$this->user = $user;
		$this->config = $config;
		$this->shout = $shout;
		$this->arcade = $arcade;
		$this->php_ext = $php_ext;
		$this->shout_table = $shout_table;
		$this->posts_table = $posts_table;

		$this->arcade_enable = $arcade !== null;
	}

	static public function getSubscribedEvents()
	{
		return array(
			'core.common'								=> 'common',
			'core.user_setup'							=> 'user_setup',
			'core.page_header'							=> 'header',
			'core.generate_smilies_after'				=> 'disable_extra_buttons',
			'core.display_custom_bbcodes'				=> 'disable_extra_buttons',
			'core.display_custom_bbcodes_modify_row'	=> 'custom_bbcodes',
			'core.submit_post_end'						=> 'submit_post_end',
			'core.delete_posts_after'					=> 'delete_posts_after',
			'core.delete_forum_content_before_query'	=> 'delete_forum_before',

			'jv.arcade.admin.delete_game'				=> 'delete_game',

			'core.permissions'							=> 'permissions_language',
			'core.viewonline_overwrite_location'		=> 'viewonline',
			'core.delete_user_after'					=> 'delete_user',

			'core.modify_posting_auth'					=> 'soft_delete_post',
			'core.mcp_global_f_read_auth_after'			=> 'soft_delete_post',
		);
	}

	public function common()
	{
		if (!defined('JV_SHOUTBOX_POSITION_TOP'))
		{
			$this->container->get('jv.shoutbox.constants');
		}
	}

	public function user_setup($event)
	{
		if ($this->config['jv_shoutbox_enable'])
		{
			$event['lang_set_ext'] = array_merge($event['lang_set_ext'], array(
				array(
					'ext_name' => 'jv/shoutbox',
					'lang_set' => 'shoutbox'
				)
			));
		}
	}

	public function header()
	{
		if (!$this->request->is_ajax() && $this->shout->enable())
		{
			$this->shout->header();

			if ($this->shout->active_page != 'custom' && $this->config['jv_shoutbox_page'] != JV_SHOUTBOX_DISPLAY_NONE && $this->check_page())
			{
				$this->shout->display();
			}
		}
	}

	public function disable_extra_buttons()
	{
		$this->shout->active_extra_buttons();
	}

	public function custom_bbcodes($event)
	{
		if (!$this->config['jv_shoutbox_custom_bbcode'] || !$this->auth->acl_get('u_jv_shoutbox_bbcode'))
		{
			return;
		}

		static $jvs_bbcode_allowed;

		if (!isset($jvs_bbcode_allowed))
		{
			$jvs_bbcode_allowed = explode("\n", $this->shout->text_config('jv_shoutbox_bbcode_allowed'));
		}

		if (!count($jvs_bbcode_allowed))
		{
			return;
		}

		if (in_array($event['row']['bbcode_tag'], $jvs_bbcode_allowed))
		{
			$this->shout->custom_bbcode = true;
			$event['custom_tags'] += array('S_JV_SHOUTBOX_ALLOW' => true);
		}
	}

	public function submit_post_end($event)
	{
		$post_data = $event['data'];

		if ($this->auth_add_message($event['mode'], $post_data['forum_id']))
		{
			$user_id = $this->user->data['user_id'];
			if ($this->config['jv_shoutbox_user_id'])
			{
				$user_id = (int) $this->shout->author('user_id', $this->config['jv_shoutbox_user_id']);
			}
			else if (!empty($this->config['jv_system_bot_user_id']))
			{
				$user_id = (int) $this->shout->author('user_id', $this->config['jv_system_bot_user_id']);
			}

			$lang_key = 'NEW_POST';

			switch ($event['mode'])
			{
				case 'post': 
					$lang_key = 'NEW_TOPIC';
				break;

				case 'quote':
					$lang_key = 'NEW_POST_QUOTE';
				break;

				case 'edit':
					$lang_key = 'EDITED_POST';
				break;
			}

			if ($user_id)
			{
				$sql_ary = array(
					'user_id'			=> $user_id,
					'user_ip'			=> '',
					'message'			=> 'JV_SHOUTBOX_F_' . $lang_key,
					'bbcode_bitfield'	=> '',
					'bbcode_uid'		=> '',
					'message_time'		=> time(),
					'post_id'			=> $post_data['post_id'],
					'uid'				=> $this->user->data['user_id']
				);

				$sql = 'INSERT INTO ' . $this->shout_table . ' ' . $this->db->sql_build_array('INSERT', $sql_ary);
				$this->db->sql_query($sql);
			}
		}
	}

	public function delete_posts_after($event)
	{
		if (!isset($event['post_ids']) || !count($event['post_ids']))
		{
			return;
		}

		$sql = 'DELETE FROM ' . $this->shout_table . '
				WHERE ' . $this->db->sql_in_set('post_id', array_map('intval', $event['post_ids']));
		$this->db->sql_query($sql);

		if ($this->db->sql_affectedrows())
		{
			$this->config->set('jv_shoutbox_last_action_time', time(), false);
		}
	}

	public function delete_forum_before($event)
	{
		if (empty($event['forum_id']))
		{
			return;
		}

		$sql = 'SELECT post_id
				FROM ' . $this->posts_table . '
				WHERE forum_id = ' . (int) $event['forum_id'];
		$result = $this->db->sql_query($sql);
		$p = array();
		while ($row = $this->db->sql_fetchrow($result))
		{
			$p['post_ids'][] = $row['post_id'];
		}
		$this->db->sql_freeresult($result);

		if (!empty($p['post_ids']))
		{
			$this->delete_posts_after($p);
		}
	}

	public function delete_game($event)
	{
		$event['tables'] = array_merge($event['tables'], array(
			$this->shout_table
		));
	}

	public function permissions_language($event)
	{
		$event['categories'] = array_merge($event['categories'], array(
			'jv_shoutbox' => 'ACL_CAT_JV_SHOUTBOX',
		));

		$event['permissions'] = array_merge($event['permissions'], array(
			// Admin Permissions
			'a_jv_shoutbox_settings'		=> array('lang' => 'ACL_A_JV_SHOUTBOX_SETTINGS'		, 'cat' => 'settings'),
			// Mod Permissions
			'm_jv_shoutbox_delete'			=> array('lang' => 'ACL_M_JV_SHOUTBOX_DELETE'		, 'cat' => 'jv_shoutbox'),
			'm_jv_shoutbox_edit'			=> array('lang' => 'ACL_M_JV_SHOUTBOX_EDIT'			, 'cat' => 'jv_shoutbox'),
			'm_jv_shoutbox_ip'				=> array('lang' => 'ACL_M_JV_SHOUTBOX_IP'			, 'cat' => 'jv_shoutbox'),
			// User Permissions
			'u_jv_shoutbox_view'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_VIEW'			, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_use'				=> array('lang' => 'ACL_U_JV_SHOUTBOX_USE'			, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_delete'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_DELETE'		, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_edit'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_EDIT'			, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_quote'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_QUOTE'		, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_flood_ignore'	=> array('lang' => 'ACL_U_JV_SHOUTBOX_FLOOD_IGNORE'	, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_bbcode'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_BBCODE'		, 'cat' => 'jv_shoutbox'),
			'u_jv_shoutbox_smilies'			=> array('lang' => 'ACL_U_JV_SHOUTBOX_SMILIES'		, 'cat' => 'jv_shoutbox')
		));
	}

	public function viewonline($event)
	{
		if ($this->shout->enable())
		{
			$this->shout->viewonline($event);
		}
	}

	public function delete_user($event)
	{
		$sql = 'DELETE FROM ' . $this->shout_table . '
				WHERE ' . $this->db->sql_in_set('user_id', array_map('intval', $event['user_ids']));
		$this->db->sql_query($sql);

		$this->config->set('jv_shoutbox_last_action_time', time(), false);
	}

	public function soft_delete_post($event)
	{
		if ($this->request->is_set_post('confirm'))
		{
			$action = $this->request->variable('action', '');
			$forum_id = (int) $this->request->variable('f', 0);
			$soft_delete = (!$this->request->is_set_post('delete_permanent') || !$this->auth->acl_get('m_delete', $forum_id)) ? true : false;

			if (($soft_delete && ($action == 'delete_topic' || $action == 'delete_post')) ||
				(!empty($event['mode']) && $event['mode'] == 'soft_delete' && !empty($event['post_id']) && !count($event['error']))
			)
			{
				$this->config->set('jv_shoutbox_last_action_time', time(), false);
			}
		}
	}

	protected function check_page()
	{
		$r = false;
		$page = $this->shout->user_page();
		$params = $this->shout->user_page(false, true);
		$arcade_page = $this->arcade_enable ? $this->arcade->root_file(false) : 'arcade';

		if (($page == 'portal' && (($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_PAGES_EXCEPT_PORTAL) || $this->shout->is_enabled_ext('jv/shoutbox_b3portal'))) ||
			($page == $arcade_page && $this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_PAGES_EXCEPT_ARCADE)
		)
		{
			return $r;
		}

		if (($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_PAGES) ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_PAGES_EXCEPT_PORTAL && $page != 'portal') ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_PAGES_EXCEPT_ARCADE && $page != $arcade_page) ||
			((!$params || $page == 'portal') && (
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_FORUM && $page == 'index') ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ARCADE && $page == $arcade_page) ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ALL_MAIN_PAGES && (in_array($page, array('portal', 'index', $arcade_page)) || strpos($page, '/') === false)))) ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_PORTAL && $page == 'portal') ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_FORUM_ALL && !in_array($page, array('portal', $arcade_page))) ||
			($this->config['jv_shoutbox_page'] == JV_SHOUTBOX_DISPLAY_ARCADE_ALL && $page == $arcade_page)
		)
		{
			$r = true;
		}

		return $r;
	}

	protected function auth_add_message($mode, $forum_id)
	{
		$r = false;

		if ($this->config['jv_shoutbox_enable'] && $this->shout->enable_forum_post())
		{
			if (($mode == 'post' && $this->config['jv_shoutbox_new_topic_post']) ||
				($mode == 'reply' && $this->config['jv_shoutbox_new_reply_post']) ||
				($mode == 'quote' && $this->config['jv_shoutbox_new_reply_quote_post']) ||
				($mode == 'edit' && $this->config['jv_shoutbox_edit_post'])
			)
			{
				$ign_forum_ids = explode(',', $this->shout->text_config('jv_shoutbox_post_ign_forums'));

				if (!in_array($forum_id, $ign_forum_ids))
				{
					$r = true;
				}
			}
		}

		return $r;
	}
}
